use jaby_engine_fconv::images::{*};
use clap::{Parser, Subcommand};
use std::path::PathBuf;
use tool_helper::Error;

#[derive(Parser)]
#[clap(about = "Converts files to various JabyEngine related file formats", long_about = None)]
struct CommandLine {
    #[clap(short='o')]
    output_file: Option<PathBuf>,

    #[clap(value_parser)]
    input_file: Option<PathBuf>,

    #[clap(subcommand)]
    sub_command: SubCommands,
}

#[derive(Subcommand)]
enum SubCommands {
    SimpleTIM(reduced_tim::Arguments)
}

fn run_main() -> Result<(), Error> {
    match CommandLine::try_parse() {
        Ok(cmd) => {
            let input  = tool_helper::open_input(cmd.input_file)?;
            let output = tool_helper::open_output(cmd.output_file)?;

            match cmd.sub_command {
                SubCommands::SimpleTIM(args) => reduced_tim::convert(args, input, output),
            }
        },
        Err(error) => Err(Error::from_error(error))
    }    
}

fn main() {
    if let Err(error) = run_main() {
        println!("{}", error.text);
        std::process::exit(error.exit_code);
    }
}