pub mod json_reader;

#[derive(Debug)]
pub struct OverlaySection {
    pub(super) name:         String,
    pub(super) file_pattern: Vec<String>
}

impl OverlaySection {
    pub fn new(name: String) -> OverlaySection {
        OverlaySection{name, file_pattern: Vec::new()}
    }

    pub fn add_file_pattern(&mut self, file_pattern: String) {
        self.file_pattern.push(file_pattern);
    }
}

#[derive(Debug)]
pub struct OverlaySlot {
    pub(super) name:      String,
    pub(super) start_adr: Option<String>,
    pub(super) sections:  Vec<OverlaySection>
}

impl OverlaySlot {
    pub fn new(name: String, start_adr: Option<&str>) -> OverlaySlot {
        let start_adr = {
            if let Some(str) = start_adr {
                Some(str.to_owned())
            }
            
            else {
                None
            }
        };
        OverlaySlot{name, start_adr, sections: Vec::new()}
    }

    pub fn set_start_adr_as_end_of(&mut self, end: &str) {
        self.start_adr = Some(format!("__{}_end", end));
    }

    pub fn add_section(&mut self, section: OverlaySection) {
        self.sections.push(section);
    }
}