use cdtypes::Error;
use std::{io::Read, fs::OpenOptions, path::PathBuf};

pub fn open_output_file(path: &str) -> Result<std::fs::File, Error> {
    Ok(convert_io_error(path, OpenOptions::new().write(true).create(true).truncate(true).open(path))?)
}

pub fn read_file_to_string(path: &str) -> Result<String, Error> {
    let mut buffer = String::new();
    if let Ok(_) = convert_io_error(path, OpenOptions::new().read(true).create(false).open(path))?.read_to_string(&mut buffer) {
        Ok(buffer)
    }

    else {
        Err(Error::GenericError(format!("Failed reading file \"{}\" to String", path)))
    }
}

pub fn read_file_to_u8(path: &str) -> Result<Vec<u8>, Error> {
    let mut buffer = Vec::default();
    if let Ok(_) = convert_io_error(path, OpenOptions::new().read(true).create(false).open(path))?.read_to_end(&mut buffer) {
        Ok(buffer)
    }

    else {
        Err(Error::GenericError(format!("Failed reading file \"{}\" to Vec<u8>", path)))
    }
}

pub fn path_to_string(path: PathBuf) -> Result<String, Error> {
    match path.into_os_string().into_string() {
        Ok(str)  => Ok(str),
        Err(_)   => Err(Error::GenericError(format!("Couldn't convert OS_String to proper string")))
    }
}

fn convert_io_error<T>(path: &str, value: Result<T, std::io::Error>) -> Result<T, Error> {
    match value {
        Ok(value)     => Ok(value),
        Err(io_error) => Err(Error::IOError(Some(path.to_string()), io_error)),
    }
}
