use cdtypes::Error;
use super::cd_desc::LBASize;

pub type EntryVec = Vec<Entry>;

pub enum Entry {
    Directory(DirectoryDataType),
    File(FileDataType)
}

pub struct DirectoryDataType {
    pub name:       String,
    pub lba:        usize,
    pub size:       usize,
    pub lba_length: usize,
    pub entries:    EntryVec,
    pub is_hidden:  bool,
}

impl DirectoryDataType {
    pub fn new(name: String, lba_size: &dyn LBASize, is_hidden: bool) -> Result<DirectoryDataType, Error> {
        Ok(DirectoryDataType{name, lba: lba_size.get_lba(), size: lba_size.get_length(), lba_length: lba_size.get_length_lba()?, entries: Vec::new(), is_hidden})
    }
}

pub struct FileDataType {
    pub name:       String,
    pub file_path:  String,
    pub lba:        usize,
    pub size:       usize,
    pub lba_length: usize,
    pub is_hidden:  bool
}

impl FileDataType {
    pub fn new(name: String, file_path: String, lba_size: &dyn LBASize, is_hidden: bool) -> Result<FileDataType, Error> {
        Ok(FileDataType{name, file_path, lba: lba_size.get_lba(), size: lba_size.get_length(), lba_length: lba_size.get_length_lba()?, is_hidden})
    }
}

pub fn new_dir_entry(entry: DirectoryDataType) -> Entry {
    Entry::Directory(entry)
}

pub fn new_file_entry(name: String, file_path: String, lba_size: &dyn LBASize, is_hidden: bool) -> Result<Entry, Error> {
    Ok(Entry::File(FileDataType::new(name, file_path, lba_size, is_hidden)?))
}