use super::super::super::types::IntermediateSector;
use cdtypes::{Error, types::{sector as raw_sector, helper::sector_count_mode2_form1}};

pub fn copy_data<'a, T, const SIZE:usize>(dst: &mut [T; SIZE], src: &'a [T]) -> &'a [T] {
    let data_size = {
        if src.len() > SIZE {
            SIZE
        }

        else {
            src.len()
        }
    };

    unsafe{std::ptr::copy_nonoverlapping(src.as_ptr(), dst.as_mut_ptr(), data_size)};
    &src[data_size..src.len()]
}

pub fn fill_sectors(sectors: &mut Vec<IntermediateSector>, mut amount: usize) {
    while amount > 0 {
        sectors.push(IntermediateSector::Empty);
        amount -= 1;
    }    
}

pub fn fill_sectors_upto(sectors: &mut Vec<IntermediateSector>, amount: usize) {
    let cur_len = sectors.len();
    if cur_len < amount {
        fill_sectors(sectors, amount - cur_len);
    }
}

pub fn write_cdxa_data(sectors: &mut Vec<IntermediateSector>, mut start_idx: usize, mut data: &[u8], mut sub_header: raw_sector::SubHeader) -> Result<(), Error> {
    let end_idx  = start_idx + sector_count_mode2_form1(data.len());

    while start_idx < end_idx {
        let mut sec_data = [0u8; raw_sector::Mode2Form1::DATA_SIZE];
        data = copy_data(&mut sec_data, data);

        if start_idx + 1 == end_idx {
            sub_header.sub_mode.set_eor();
            sub_header.sub_mode.set_eof();            
        }

        if let Some(sector) = sectors.get_mut(start_idx) {
            *sector    = IntermediateSector::CDXAData(sec_data, sub_header.clone());
            start_idx += 1;
        }

        else {
            return Err(Error::NoSectorForLBA(start_idx));
        }
    }

    Ok(())
}

pub fn write_cdxa_audio(sectors: &mut Vec<IntermediateSector>, mut start_idx: usize, data: &Vec<(raw_sector::SubHeader, [u8; raw_sector::Mode2Form2::DATA_SIZE])>) -> Result<(), Error> {
    for data in data {
        if let Some(sector) = sectors.get_mut(start_idx) {
            *sector    = IntermediateSector::CDXAAudio(data.1, data.0.clone());
            start_idx += 1;
        }

        else {
            return Err(Error::NoSectorForLBA(start_idx));
        }
    }

    Ok(())
}