use cdtypes::types::sector::{Mode2Form1, Mode2Form2, SubHeader};
use super::IntermediateSector;

pub struct SectorBuilder{}
pub struct SubHeaderBuilder{}
pub struct EmptySectorBuilder{}

#[allow(non_snake_case)]
impl SectorBuilder {
    pub fn CDXAData<T>(data: T, sub_header: SubHeader) -> IntermediateSector {
        unsafe {
            let ref_data = std::mem::transmute::<&T, &[u8; Mode2Form1::DATA_SIZE]>(&data);
            IntermediateSector::CDXAData(*ref_data, sub_header)
        }
    }
}

#[allow(non_snake_case)]
impl SubHeaderBuilder {
    pub fn VolumeDescriptorDefault(is_eof: bool) -> SubHeader {
        let mut sub_header = SubHeader::default();

        sub_header.sub_mode.set_eor();
        sub_header.sub_mode.set_data();
        
        if is_eof {
            sub_header.sub_mode.set_eof();
        }

        sub_header
    }

    pub fn DirectoryRecordDefault(is_last: bool) -> SubHeader {
        let mut sub_header = SubHeader::default();

        sub_header.sub_mode.set_data();
        if is_last {
            sub_header.sub_mode.set_eor();
            sub_header.sub_mode.set_eof();
        }
        sub_header
    }

    pub fn DataDefault(is_last: bool) -> SubHeader {
        Self::DirectoryRecordDefault(is_last)
    }
}

#[allow(non_snake_case)]
impl EmptySectorBuilder {
    pub fn CDXAData() -> IntermediateSector {
        IntermediateSector::CDXAData([0u8; Mode2Form1::DATA_SIZE], SubHeaderBuilder::DataDefault(false))
    }

    pub fn CDXAAudio() -> IntermediateSector {
        let sub_header = {
            let mut sub_header = SubHeader::default_form2();

            sub_header.sub_mode.clear_audio();
            sub_header
        };
        IntermediateSector::CDXAAudio([0u8; Mode2Form2::DATA_SIZE], sub_header)
    }
}